searchData={"items":[{"type":"module","title":"Airframe","doc":"Airframe is an authorization library.\n\nTo use Airframe, you must implement the `Airframe.Policy` behaviour.\n\nIn your policy module `use Airframe.Policy`:\n\n    defmodule MyApp.MyContext.MyPolicy do\n      use Airframe.Policy\n\n      @spec allow(subject, action, actor)\n      def allow(_subject_, _action, _actor)do\n        true # allow everything by anyone!\n      end\n    end\n\nTo check against a policy, you use `Airframe.check/4`\n\n    def delete(subject, opts) do\n      # subject - the object to be acted upon\n      # action - the action to be performed on the subject\n      # actor - the current authentication, e.g. conn.assigns.current_user, or an API key, etc\n      # policy - the module that implements the `Airframe.Policy` behaviour to check against\n      with {:ok, subject} <- Airframe.check(subject, action, actor, policy) do\n        # actor is allowed to perform action on subject according to policy.\n      end\n    end\n\nOr `Airframe.check!/4`:\n\n    def list(opts) do\n      Post\n      |> Airframe.check!(:read, opts[:current_user], MyPolicy)\n      |> Repo.all()\n    end","ref":"Airframe.html"},{"type":"type","title":"Airframe.action/0","doc":"The action to be performed on the subject.\n\nThis is typically an atom that represents the action to be performed,\nsuch as `:create`, `:read`, `:update`, `:delete`, however it can be any value.","ref":"Airframe.html#t:action/0"},{"type":"type","title":"Airframe.actor/0","doc":"The actor of the action.\n\nThis is typically the current user, session token, or some other\nvalue that represents the actor of the action.","ref":"Airframe.html#t:actor/0"},{"type":"macro","title":"Airframe.check/3","doc":"Macro version of `Airframe.check/4`.\n\nInfers the policy from the calling module,\nand the action from the calling function name.","ref":"Airframe.html#check/3"},{"type":"macro","title":"Example - Airframe.check/3","doc":"defmodule MyApp.MyContext do\n      use Airframe.Policy\n      # ...\n      def create(attr, opts) do\n        # infer the action to be the name of the calling function (`create`)\n        # and the policy to be the calling module (`MyApp.MyContext`)\n        changeset = %Post{} |> Post.changeset(attr)\n        with {:ok, changeset} <- Airframe.check(changeset, actor) do\n          # actor is allowed to perform action on changeset according to policy.\n        end\n      end\n    end","ref":"Airframe.html#check/3-example"},{"type":"function","title":"Airframe.check/4","doc":"","ref":"Airframe.html#check/4"},{"type":"macro","title":"Airframe.check!/3","doc":"Macro version of `Airframe.check!/4`.\n\nInfers the policy from the calling module,\nand the action from the calling function name.","ref":"Airframe.html#check!/3"},{"type":"function","title":"Airframe.check!/4","doc":"","ref":"Airframe.html#check!/4"},{"type":"type","title":"Airframe.policy/0","doc":"The policy module that implements the `Airframe.Policy` behaviour.","ref":"Airframe.html#t:policy/0"},{"type":"type","title":"Airframe.subject/0","doc":"The subject of the action.\n\nFor \"read\" actions, this is typically a schema or query that the policy\nis expected to narrow down access to.\n\nFor \"write\" actions, this is typically a struct or changeset that the policy\nis expected to validate authorization for.","ref":"Airframe.html#t:subject/0"},{"type":"behaviour","title":"Airframe.Policy","doc":"The Policy behaviour, using macro, and defaults.","ref":"Airframe.Policy.html"},{"type":"behaviour","title":"Examples - Airframe.Policy","doc":"defmodule MyApp.Policy do\n      use Airframe.Policy\n\n      def allow(%User{id: user_id}, :update, %{current_user: %User{id: user_id}}), do: true\n      def allow(%User{id: user_id}, :update, %{current_user: %User{id: other_id}}), do: false\n    end","ref":"Airframe.Policy.html#module-examples"},{"type":"macro","title":"Airframe.Policy.__using__/1","doc":"Set up the module as an `Airframe.Policy`.","ref":"Airframe.Policy.html#__using__/1"},{"type":"macro","title":"Options - Airframe.Policy.__using__/1","doc":"* `defer` - If provided, it is the module to defer to for policy checking instead of this module.\n* `default` - If provided, a catch-all `allow/3` is appended that always returns this value.\n  This is useful for policies that always allow or deny everything by default. If not provided,\n  the user is responsible for handling all cases in their policy module.","ref":"Airframe.Policy.html#__using__/1-options"},{"type":"macro","title":"Example - Airframe.Policy.__using__/1","doc":"defmodule MyApp.Context do\n      # simply\n      use Airframe.Policy\n\n      # or if you want to implement `c:Airframe.Policy.allow/3` in a different module:\n      use Airframe.Policy, defer: MyApp.Context.Policy\n    end","ref":"Airframe.Policy.html#__using__/1-example"},{"type":"callback","title":"Airframe.Policy.allow/3","doc":"Checks if the `action` is allowed on `subject` by `actor`.\n\nCan return plain true/false, or a tuple `{:ok, scoped_subject}` to still allow the action,\nwhile narrowing the scope of the original subject.\nThis is useful if the original subject is an `Ecto.Query` or similar that can be narrowed down.\nCan also return `{:error, reason :: any()}` which will be passed along to the caller.","ref":"Airframe.Policy.html#c:allow/3"},{"type":"function","title":"Airframe.Policy.check/4","doc":"Checks if the `action` is allowed on `subject` for `actor` by `policy`.\n\nIf the `ecto_changeset` feature flag is enabled (default: true),\nthis function will return `{:error, changeset}` if the `subject` is an `Ecto.Changeset` and is marked as invalid.","ref":"Airframe.Policy.html#check/4"},{"type":"function","title":"Airframe.Policy.check!/4","doc":"Checks if the `action` is allowed on `subject` for `actor` by `policy`.\n\nLike `check/4`, but raises an `Airframe.UnauthorizedError` if the action is not allowed.","ref":"Airframe.Policy.html#check!/4"},{"type":"exception","title":"Airframe.PolicyError","doc":"","ref":"Airframe.PolicyError.html"},{"type":"exception","title":"Airframe.UnauthorizedError","doc":"","ref":"Airframe.UnauthorizedError.html"},{"type":"function","title":"Airframe.UnauthorizedError.message/1","doc":"","ref":"Airframe.UnauthorizedError.html#message/1"},{"type":"extras","title":"Airframe","doc":"# Airframe\n\n[![CI](https://github.com/tudborg/airframe/actions/workflows/elixir.yml/badge.svg)](https://github.com/tudborg/airframe/actions/workflows/elixir.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/airframe.svg)](https://hex.pm/packages/airframe)\n[![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/airframe/)\n\nAirframe is an authorization library.\n\nDocumentation can be found at  .\n\nAt it's core is the `Airframe.Policy` that you implement for all relevant \"subjects\".  \nA policy is a module implemetning `c:Airframe.Policy.allow/3`, where you implement checks against a subject and action for some \"actor\" like a `current_user` or session token.\n\nIt differentiates itself from other similar libraries by allowing the policy to change the subject, like:\n- Changing an `Ecto.Query` to narrow done the scope to the current user.\n- Removing changes from an `Ecto.Changeset` based on the current user's access level.\n- Completely replacing the subject with an alternative subject.\n- \nThis makes it possible to combine access scoping and action checking in a single policy.\n\nIt can be used anywhere in your application, but is intended to be used in your context modules to\nensure that authorization is handled at the same abstraction level everywhere, regardless of interface.","ref":"readme.html"},{"type":"extras","title":"Installation - Airframe","doc":"The package can be installed by adding `airframe` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:airframe, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Examples - Airframe","doc":"You can define a Policy directly in a context:\n\n```elixir\ndefmodule MyApp.MyContext do\n  use Airframe.Policy\n\n  @spec allow(subject, action, actor)\n  def allow(_subject_, _action, _actor)do\n    true # allow everything by anyone!\n  end\nend\n```\n\nOr more typically in a larger context module, you defer the policy to it's own module:\n\n```elixir\ndefmodule MyApp.MyContext do\n  use Airframe.Policy, defer: MyApp.MyContext.MyPolicy\n\n  # ...\nend\n\ndefmodule MyApp.MyContext.MyPolicy do\n  use Airframe.Policy\n\n  @spec allow(subject, action, actor)\n  def allow(_subject_, _action, _actor)do\n    true # allow everything by anyone!\n  end\nend\n```","ref":"readme.html#examples"},{"type":"extras","title":"Authorization - Airframe","doc":"You check against a policy with `Airframe.check/4`\n\n```elixir\nwith {:ok, subject} <- Airframe.check(subject, action, actor, policy) do\n  # actor is allowed to perform action on subject according to policy.\nend\n```\n\nand it's `raise`ing version `Airframe.check!/4`:\n\n```elixir\nAirframe.check(subject, action, actor, policy)\n```\n\nwhich is very convenient if your subject is a schema module or ecto query:\n\n```elixir\ndef list(opts) do\n  Post\n  |> Airframe.check!(:read, opts[:current_user], MyPolicy)\n  |> Repo.all()\nend\n```\n\nThere is an `Airframe.check/2` and `Airframe.check!/2` macro\nthat will infer the policy and (optionally) the action from the calling context.\n\n```elixir\ndefmodule MyApp.MyContext do\n  use Airframe.Policy\n\n  def allow(_subject_, _action, _actor) do\n    true # allow everything by anyone!\n  end\n\n  def create(attr, opts) do\n    # infer the action to be the name of the calling function (`create`)\n    # and the policy to be the calling module (`MyApp.MyContext`)\n    # By using `Airframe.Policy` you automatically require Airframe\n    # such that the `Airframe.check/2` macro is available\n    with {:ok, subject} <- Airframe.check(subject, actor) do\n      # actor is allowed to perform action on subject according to policy.\n    end\n  end\nend\n```","ref":"readme.html#authorization"},{"type":"extras","title":"Full Example - Airframe","doc":"```elixir\ndefmodule MyApp.MyContext do\n  # Allow this context to be used as a policy,\n  # but defer to it's own policy module for the callback function.\n  use Airframe.Policy, defer: __MODULE__.Policy\n\n  def list(opts) do\n    # we want to list Post\n    Post\n    # we check that current_user is allowed to do so\n    # (the action is derived from the calling function - `list` in this case,\n    # you can ofc. specify the action manually as the second argument to check!/3)\n    |> Airframe.check!(opts[:current_user])\n    |> Repo.all()\n  end\n\n  def get!(id, opts) do\n    Post\n    |> Airframe.check!(opts[:current_user])\n    |> Repo.get!(id)\n  end\n  \n  def create(attrs, opts) do\n    with {:ok, attrs} <- Airframe.check(attrs, opts[:current_user]) do\n      %Post{}\n      |> Post.changeset(attrs)\n      |> Repo.insert()\n    end\n  end\n\n  def update(post, attrs, opts) do\n    with {:ok, {post, attrs}} <- Airframe.check({post, attrs}, opts[:current_user]) do\n      post\n      |> Post.changeset(attrs)\n      |> Repo.update()\n    end\n  end\n\n  # Airframe has special support for Ecto.Changeset.\n  # If the subject is a changeset, it will only ever be checked against the policy\n  # if it is valid. Otherwise `{:error, changeset}` is returned.\n\n  def update(post, attrs, opts) do\n    changeset = Post.changeset(post, attrs)\n\n    with {:ok, changeset} <- Airframe.check(changeset, opts[:current_user]) do\n      Repo.update(changeset)\n    end\n  end\n\n  # ...\nend\n\ndefmodule MyApp.MyContext.Policy do\n  # this is the actual policy module.\n  use Airframe.Policy\n\n  # allow the `list` and `get` action on `Post`,\n  # but we narrow the scope to only posts from the actor (User, in this case)\n  @spec allow(subject, action, actor)\n  def allow(Post, action, %User{id: user_id}) when action in [:list, :get] do\n    {:ok, from(p in Post, where: p.user_id == ^user_id)}\n  end\n\n  # allow a user to create a post if the post attrs user_id matches their user's id\n  def allow(attrs, :create, %User{id: user_id}) do\n    attrs[\"user_id\"] == user_id\n  end\n\n  # allow a user to update a post if the post is their own.\n  def allow({%Post{user_id: user_id}, attrs}, :update, %User{id: user_id}), do: true\n  def allow({%Post{user_id: _}, attrs}, :update, %User{}), do: false\n\n  # or if you'd rather check on the changeset itself:\n  def allow(%Ecto.Changeset{data: %Post{user_id: user_id}}, :update, %User{id: user_id}), do: true\n  def allow(%Ecto.Changeset{data: %Post{user_id: _}}, :update, %User{id: _}), do: false\n\n  # having the changeset as the subject also makes it easy to modify it:\n  def allow(%Ecto.Changeset{data: %Post{user_id: user_id}}=changeset, :update, %User{id: user_id}) do\n    {:ok, Ecto.Changeset.delete_change(changeset, :approved_by_id)}\n  end\n\n\n  # ...\nend\n```","ref":"readme.html#full-example"},{"type":"extras","title":"Changelog for v0.x","doc":"# Changelog for v0.x","ref":"changelog.html"},{"type":"extras","title":"v0.1.3 - Changelog for v0.x","doc":"","ref":"changelog.html#v0-1-3"},{"type":"extras","title":"Added - Changelog for v0.x","doc":"- `plug_status: :unauthorized` on the `Airframe.UnauthorizedError`","ref":"changelog.html#added"},{"type":"extras","title":"v0.1.2 - Changelog for v0.x","doc":"","ref":"changelog.html#v0-1-2"},{"type":"extras","title":"Fixed - Changelog for v0.x","doc":"- documentation.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.1.1 - Changelog for v0.x","doc":"Initial release.","ref":"changelog.html#v0-1-1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.1"}}